@@ -213,8 +213,12 @@ impl<'a> SignedDataBuilder<'a> {
         self
     }
 
-    /// Construct a BER-encoded ASN.1 document containing a `SignedData` object.
-    pub fn build_ber(&self) -> Result<Vec<u8>, CmsError> {
+    /// Construct a DER-encoded ASN.1 document containing a `SignedData` object.
+    ///
+    /// RFC 5652 says `SignedData` is BER encoded. However, DER is a stricter subset
+    /// of BER. DER encodings are valid BER. So producing DER encoded data is perfectly
+    /// valid. We choose to go with the more well-defined encoding format.
+    pub fn build_der(&self) -> Result<Vec<u8>, CmsError> {
         let mut signer_infos = SignerInfos::default();
         let mut seen_digest_algorithms = HashSet::new();
         let mut seen_certificates = self.certificates.clone();
@@ -305,25 +309,22 @@ impl<'a> SignedDataBuilder<'a> {
                 signed_attributes_data: None,
             };
 
-            // The signature is computed over content embedded in the final message.
-            // This content is the optional encapsulated content plus the DER
-            // serialized signed attributes.
-            let mut signed_content = Vec::new();
-            if let Some(encapsulated) = &self.signed_content {
-                signed_content.extend(encapsulated);
-            }
-            if let Some(attributes_data) = signer_info.signed_attributes_digested_content()? {
-                signed_content.extend(attributes_data);
-            }
+            // The content being signed is the DER encoded signed attributes, if present, or the
+            // encapsulated content. Since we always create signed attributes above, it *must* be
+            // the DER encoded signed attributes.
+            let signed_content = signer_info
+                .signed_attributes_digested_content()?
+                .expect("presence of signed attributes should ensure this is Some(T)");
 
             let (signature, signature_algorithm) = signer.signing_key.sign(&signed_content)?;
 
-            signer_info.signature = SignatureValue::new(Bytes::from(signature));
+            signer_info.signature = SignatureValue::new(Bytes::from(signature.clone()));
             signer_info.signature_algorithm = signature_algorithm.into();
 
             if let Some(url) = &signer.time_stamp_url {
+                // The message sent to the TSA (via a digest) is the signature of the signed data.
                 let res =
-                    time_stamp_message_http(url.clone(), &signed_content, signer.digest_algorithm)?;
+                    time_stamp_message_http(url.clone(), &signature, signer.digest_algorithm)?;
 
                 if !res.is_success() {
                     return Err(TimeStampError::Unsuccessful(res.clone()).into());
@@ -376,11 +377,10 @@ impl<'a> SignedDataBuilder<'a> {
             digest_algorithms,
             content_info: EncapsulatedContentInfo {
                 content_type: self.content_type.clone(),
-                content: if let Some(content) = &self.signed_content {
-                    Some(OctetString::new(Bytes::copy_from_slice(content)))
-                } else {
-                    None
-                },
+                content: self
+                    .signed_content
+                    .as_ref()
+                    .map(|content| OctetString::new(Bytes::copy_from_slice(content))),
             },
             certificates: if certificates.is_empty() {
                 None
@@ -394,7 +394,7 @@ impl<'a> SignedDataBuilder<'a> {
         let mut ber = Vec::new();
         signed_data
             .encode_ref()
-            .write_encoded(Mode::Ber, &mut ber)?;
+            .write_encoded(Mode::Der, &mut ber)?;
 
         Ok(ber)
     }
@@ -408,7 +408,7 @@ mod tests {
         x509_certificate::{testutil::*, EcdsaCurve},
     };
 
-    const APPLE_TIMESTAMP_URL: &str = "http://timestamp.apple.com/ts01";
+    const DIGICERT_TIMESTAMP_URL: &str = "http://timestamp.digicert.com";
 
     #[test]
     fn simple_rsa_signature() {
@@ -420,7 +420,7 @@ mod tests {
         let ber = SignedDataBuilder::default()
             .signed_content(vec![42])
             .signer(signer)
-            .build_ber()
+            .build_der()
             .unwrap();
 
         let signed_data = crate::SignedData::parse_ber(&ber).unwrap();
@@ -443,13 +443,13 @@ mod tests {
         let cert = rsa_cert();
 
         let signer = SignerBuilder::new(&key, cert)
-            .time_stamp_url(APPLE_TIMESTAMP_URL)
+            .time_stamp_url(DIGICERT_TIMESTAMP_URL)
             .unwrap();
 
         let ber = SignedDataBuilder::default()
             .signed_content(vec![42])
             .signer(signer)
-            .build_ber()
+            .build_der()
             .unwrap();
 
         let signed_data = crate::SignedData::parse_ber(&ber).unwrap();
@@ -458,6 +458,18 @@ mod tests {
             let unsigned = signer.unsigned_attributes().unwrap();
             let tst = unsigned.time_stamp_token.as_ref().unwrap();
             assert!(tst.certificates.is_some());
+
+            let tst_signed_data = signer.time_stamp_token_signed_data().unwrap().unwrap();
+            for signer in tst_signed_data.signers() {
+                signer
+                    .verify_message_digest_with_signed_data(&tst_signed_data)
+                    .unwrap();
+                signer
+                    .verify_signature_with_signed_data(&tst_signed_data)
+                    .unwrap();
+            }
+
+            assert!(signer.verify_time_stamp_token().unwrap().is_some());
         }
     }
 
@@ -470,7 +482,7 @@ mod tests {
                 .signed_content("hello world".as_bytes().to_vec())
                 .certificate(cert.clone())
                 .signer(SignerBuilder::new(&key, cert))
-                .build_ber()
+                .build_der()
                 .unwrap();
 
             let signed_data = SignedData::parse_ber(&cms).unwrap();
@@ -491,7 +503,7 @@ mod tests {
             .signed_content("hello world".as_bytes().to_vec())
             .certificate(cert.clone())
             .signer(SignerBuilder::new(&key, cert))
-            .build_ber()
+            .build_der()
             .unwrap();
 
         let signed_data = SignedData::parse_ber(&cms).unwrap();
